#winSpeechRecognission.py
#Written by Jason Custer (jscuster@gmail.com)
#This file is covered by the GNU General Public License.
#copyright 2012, Jason Custer
#It is my great pleasure to donate this file to the writers and users of NVDA, the best screen reader for windows.

import globalPluginHandler
import api
from NVDAObjects.window import Window

class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(GlobalPlugin, self).__init__()
		self.feedbackWindow=None #the recognizer feedback window
		self.keepSpeakingFeedbackStatus = False #boolean to toggle whether to automatically speak feedback updates

	#search the children of the NVDA object for the child named "name"
	def getChildrenByName(self, root, name):
		return [i for i in root.children if i.name == name]

	#get the feedback window when the user requests its status
	#This takes time, so we store it once we get it.
	def getFeedbackWindow(self):
		if self.feedbackWindow == None: #if we havn't gotten it
			topWindowName = u'Windows Speech Recognition' #constant window names 
			feedbackWindowName = u'Speech Recognition feedback'
			tmp = self.getChildrenByName(api.getDesktopObject(), topWindowName) #get the root recognizer
			if len(tmp) > 0: #make sure we found it
				tmp = self.getChildrenByName(tmp[0], topWindowName) #the subwindow, has the same name.
				if len(tmp) > 0: #make sure we found it.
					fbwin=self.getChildrenByName(tmp[0].children[1], feedbackWindowName) # the window of interest
					if len(fbwin) > 0: #confirm we got it
						self.feedbackWindow = fbwin[0] #store it
		return self.feedbackWindow #return it

	#speak the recognizer's feedback window
	def script_speakRecognizerStatus(self, gesture):
		self.speakFeedbackStatus()
	script_speakRecognizerStatus.__doc__ = _("Speaks the feedback status of the speech recognission window if active.")

	#general func to speak the contents of the feedback window.
	def speakFeedbackStatus(self):
		w=self.getFeedbackWindow()
		import ui
		if w == None:
			ui.message("recognizer not found.")
		else:
			ui.message(w.value)

	def script_toggleStatusUpdates(self, gesture):
		self.keepSpeakingFeedbackStatus = not self.keepSpeakingFeedbackStatus #toggle the flag
		import ui
		if self.keepSpeakingFeedbackStatus:
			ui.message(_("speak recognition status updates on."))
		else:
			ui.message(_("speak recognition status updates off."))
	script_toggleStatusUpdates.__doc__ = _("Toggles automatic speaking of the speech recognizer feedback window.")

	#event fires when the feedback window changes
	def event_valueChange(self, obj, next):
		fb=self.getFeedbackWindow()
		if fb != None: #if the recognizer is open
			if obj.name == fb.name:
				if self.keepSpeakingFeedbackStatus: 
					self.speakFeedbackStatus()

	#register the keyboard commands.
	__gestures = {"kb:NVDA+shift+r":"toggleStatusUpdates", "kb:NVDA+r":"speakRecognizerStatus",}